/* stim.cc Parallel Port Stimulator */

#include <X11/Xmu/Editres.h>
#include <Xm/MainW.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/SeparatoG.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>
#include <Xm/TextF.h>
#include <iostream.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include "icon.xbm"
#include "../modules/control.h"

#define VERSION "0.1"

/* global variables */

/* Structure for menu system */
typedef struct {
  Widget f_menu;
  Widget l_setup;
  Widget s_setup;
  Widget quit;
  Widget about;
} f_menu;

typedef struct {
  Widget r_menu;
  Widget s_run;
  Widget e_run;
} r_menu;

typedef struct {
  f_menu file;
  r_menu run;
} main_menu;

typedef struct {
  Widget tb[8];
  Widget tf_p[8];
  Widget tf_tp[8];
  Widget tf_td[8];
  Widget tf_pp[8];
  Widget tf_pd[8];
} user_interface;

XtAppContext          app;
Widget                main_w, stim_label;
main_menu             mymenu;
int                   is_running; /* stimulator is on */
int                   fd;         /* communication channel FIFO to RT-TASK */
struct my_msg_struct  msg;        /* parameters for stimulation */
struct user_interface ui;         /* user interface */

main(int argc, char *argv[]) {
  Pixmap    bitmap;
  Pixel     fg, bg;
  Widget    toplevel, mess_a, rowcol_w;
  Widget    menubar, PullDown;
  XmString  label;
  int       i;
  char      my_text[256];

  extern void f_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void r_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void setmsg_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void update_ui();

  /* Initialize */
  is_running=0;
  /* Initialize my_msg_struct */
  /* Channel 0 */
  msg.is_active[0]=1; msg.phase[0]=0;
  msg.pulse_d[0]=2; msg.pulse_l[0]=50;
  msg.train_d[0]=200; msg.train_l[0]=2000; 
  /* Channel 1 */
  msg.is_active[1]=1; msg.phase[1]=1000;
  msg.pulse_d[1]=2; msg.pulse_l[1]=50;
  msg.train_d[1]=200; msg.train_l[1]=2000; 
  /* Channel 2 */
  msg.is_active[2]=1; msg.phase[2]=1500;
  msg.pulse_d[2]=2; msg.pulse_l[2]=50;
  msg.train_d[2]=200; msg.train_l[2]=2000; 
  /* Channel 3 */
  msg.is_active[3]=0; msg.phase[3]=0;
  msg.pulse_d[3]=1; msg.pulse_l[3]=50;
  msg.train_d[3]=200; msg.train_l[3]=2000; 
  /* Channel 4 */
  msg.is_active[4]=0; msg.phase[4]=0;
  msg.pulse_d[4]=1; msg.pulse_l[4]=50;
  msg.train_d[4]=200; msg.train_l[4]=2000; 
  /* Channel 5 */
  msg.is_active[5]=0; msg.phase[5]=0;
  msg.pulse_d[5]=1; msg.pulse_l[5]=50;
  msg.train_d[5]=200; msg.train_l[5]=2000; 
  /* Channel 6 */
  msg.is_active[6]=0; msg.phase[6]=0;
  msg.pulse_d[6]=1; msg.pulse_l[6]=50;
  msg.train_d[6]=200; msg.train_l[6]=2000; 
  /* Channel 7 */
  msg.is_active[7]=0; msg.phase[7]=0;
  msg.pulse_d[7]=1; msg.pulse_l[7]=50;
  msg.train_d[7]=200; msg.train_l[7]=2000; 

  /* open FIFO */
  fd=open("/dev/rtf10", O_WRONLY);

  toplevel=XtVaAppInitialize(&app, "XmPStim", NULL, 0,
			     &argc, argv, NULL, NULL);

  XtVaGetValues(toplevel, XmNforeground, &fg, XmNbackground, &bg, NULL);
  bitmap=XCreatePixmapFromBitmapData(XtDisplay(toplevel),
				     RootWindowOfScreen(XtScreen(toplevel)),
				     icon_bits, icon_width, icon_height,
				     fg, bg, DefaultDepthOfScreen(XtScreen(toplevel)));
  XtVaSetValues(toplevel, 
		XmNiconPixmap, bitmap,
		XmNtitle, "XmPStim v"VERSION,
		NULL);
  XtAddEventHandler(toplevel, (EventMask)0, True,
		    _XEditResCheckMessages, NULL);

  /*   m a i n w i n d o w    */
  main_w=XtVaCreateWidget("main_w", xmMainWindowWidgetClass, toplevel,
			  NULL);
  /*   m e s s a g e a r e a   */
  mess_a=XtVaCreateWidget("frame", xmFrameWidgetClass, main_w,
				 XmNshadowType, XmSHADOW_OUT,
				 NULL);
  XtVaSetValues(main_w, XmNmessageWindow, mess_a, NULL);
  stim_label=XtVaCreateManagedWidget("mess_l", xmLabelWidgetClass, mess_a,
				     XmNlabelString, XmStringCreateLocalized("not running"),
				     XmNrecomputeSize, False,
				     NULL);
  XtManageChild(mess_a);

  /*   r o w   -   c o l u m n    f o r  input of parameters   */
  rowcol_w=XtVaCreateManagedWidget("rowcol_w",xmRowColumnWidgetClass, main_w,
				   XmNorientation, XmHORIZONTAL,
				   XmNnumColumns, 9,
				   XmNpacking, XmPACK_COLUMN,
				   NULL);
  XtVaCreateManagedWidget("Activate", xmLabelWidgetClass, rowcol_w,
			  XmNlabelString, XmStringCreateLocalized("Activate"),
			  NULL);
  XtVaCreateManagedWidget("Phase", xmLabelWidgetClass, rowcol_w,
			  XmNlabelString, XmStringCreateLocalized("Phase (ms)"),
			  NULL);
  XtVaCreateManagedWidget("Train Periode", xmLabelWidgetClass, rowcol_w,
			  XmNlabelString, XmStringCreateLocalized("Train Periode (ms)"),
			  NULL);
  XtVaCreateManagedWidget("Train Duration", xmLabelWidgetClass, rowcol_w,
			  XmNlabelString, XmStringCreateLocalized("Train Duration (ms)"),
			  NULL);
  XtVaCreateManagedWidget("Pulse Periode", xmLabelWidgetClass, rowcol_w,
			  XmNlabelString, XmStringCreateLocalized("Pulse Periode (ms)"),
			  NULL);
  XtVaCreateManagedWidget("Pulse Duration", xmLabelWidgetClass, rowcol_w,
			  XmNlabelString, XmStringCreateLocalized("Pulse Duration (ms)"),
			  NULL);
  for (i=0; i<8; i++) {
    sprintf(my_text, "Channel %d", i+1);
    ui.tb[i]=XtVaCreateManagedWidget(my_text, xmToggleButtonWidgetClass, rowcol_w,
				     XmNuserData, i,
				     NULL);
    XtAddCallback(ui.tb[i], XmNvalueChangedCallback, setmsg_cb, (XtPointer)0);
    ui.tf_p[i]=XtVaCreateManagedWidget("phase", xmTextFieldWidgetClass, rowcol_w,
				       XmNcolumns, 10,
				       XmNuserData, i,
				       NULL);
    XtAddCallback(ui.tf_p[i], XmNlosingFocusCallback, setmsg_cb, (XtPointer)1);
    ui.tf_tp[i]=XtVaCreateManagedWidget("t-p", xmTextFieldWidgetClass, rowcol_w,
					XmNcolumns, 10,
					XmNuserData, i,
					NULL);
    XtAddCallback(ui.tf_tp[i], XmNlosingFocusCallback, setmsg_cb, (XtPointer)2);
    ui.tf_td[i]=XtVaCreateManagedWidget("t-d", xmTextFieldWidgetClass, rowcol_w,
					XmNcolumns, 10,
					XmNuserData, i,
					NULL);
    XtAddCallback(ui.tf_td[i], XmNlosingFocusCallback, setmsg_cb, (XtPointer)3);
    ui.tf_pp[i]=XtVaCreateManagedWidget("p-p", xmTextFieldWidgetClass, rowcol_w,
					XmNcolumns, 10,
					XmNuserData, i,
					NULL);
    XtAddCallback(ui.tf_pp[i], XmNlosingFocusCallback, setmsg_cb, (XtPointer)4);
    ui.tf_pd[i]=XtVaCreateManagedWidget("p-d", xmTextFieldWidgetClass, rowcol_w,
					XmNcolumns, 10,
					XmNuserData, i,
					NULL);
    XtAddCallback(ui.tf_pd[i], XmNlosingFocusCallback, setmsg_cb, (XtPointer)5);
  }
  update_ui();
  XtManageChild(main_w);

  /*   M e n u s y s t e m   */
  menubar = XmCreateMenuBar(main_w, "menubar",NULL,0);
  /*   F i l e m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "FilePullDown", NULL,0);
  mymenu.file.f_menu=XtVaCreateManagedWidget("File",
			  xmCascadeButtonWidgetClass, menubar,
			  XmNlabelString, XmStringCreateLocalized("File"),
			  XmNmnemonic, 'F',
			  XmNsubMenuId, PullDown,
			  NULL);
  mymenu.file.l_setup=XtVaCreateManagedWidget("Load Setup",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'L',
					      NULL);
  XtAddCallback(mymenu.file.l_setup, XmNactivateCallback, f_cb, (XtPointer)0);
  mymenu.file.s_setup=XtVaCreateManagedWidget("Save Setup",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'S',
					      NULL);
  XtAddCallback(mymenu.file.s_setup, XmNactivateCallback, f_cb, (XtPointer)1);
  XtVaCreateManagedWidget("separator", xmSeparatorGadgetClass, PullDown, NULL);
  mymenu.file.about=XtVaCreateManagedWidget("About",
					    xmPushButtonGadgetClass, PullDown,
					    XmNmnemonic, 'A',
					    NULL);
  XtAddCallback(mymenu.file.about, XmNactivateCallback, f_cb, (XtPointer)2);
  XtVaCreateManagedWidget("separator", xmSeparatorGadgetClass, PullDown, NULL);
  mymenu.file.quit=XtVaCreateManagedWidget("Quit",
					   xmPushButtonGadgetClass, PullDown,
					   XmNmnemonic, 'Q',
					   NULL);
  XtAddCallback(mymenu.file.quit, XmNactivateCallback, f_cb, (XtPointer)3);

  /*   R u n m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "RunPullDown", NULL,0);
  mymenu.run.r_menu=XtVaCreateManagedWidget("Run",
			  xmCascadeButtonWidgetClass, menubar,
			  XmNlabelString, XmStringCreateLocalized("Run"),
			  XmNmnemonic, 'R',
			  XmNsubMenuId, PullDown,
			  NULL);
  mymenu.run.s_run=XtVaCreateManagedWidget("Start Stimulation",
					   xmPushButtonGadgetClass, PullDown,
					   XmNmnemonic, 'S',
					   NULL);
  XtAddCallback(mymenu.run.s_run, XmNactivateCallback, r_cb, (XtPointer)0);
  mymenu.run.e_run=XtVaCreateManagedWidget("End Stimulation",
					   xmPushButtonGadgetClass, PullDown,
					   XmNmnemonic, 'E',
					   NULL);
  XtAddCallback(mymenu.run.e_run, XmNactivateCallback, r_cb, (XtPointer)1);

  XtSetSensitive(mymenu.run.e_run,False);
  XtManageChild(menubar);

  XmMainWindowSetAreas(main_w, menubar, NULL, NULL, NULL, rowcol_w);

  XtRealizeWidget(toplevel);
  XtAppMainLoop(app);
}
